#ifndef COMMON_H
#define COMMON_H

/* common typedefs, defines, strings and vars... */

typedef unsigned char Byte;
typedef unsigned char Flag;
typedef unsigned char Boolean;
typedef signed char SignedByte;

#define False ((Byte) 0)
#define True ((Byte) 1) 
#define None ((Byte) 0)
#define High ((Byte) 1) 
#define Low ((Byte) 0) 
#define ByteLogicalHigh ((Byte) 255)
#define ByteLogicalLow ((Byte) 0)
#define PortOn ((Byte) 255)
#define PortOff ((Byte) 0)
#define Yes ((Byte) 1)
#define No ((Byte) 0)
#define On ((Byte) 1)
#define Off ((Byte) 0)
#define Plus ((Byte) 1)
#define Minus ((Byte) 0)
#define Positive ((Byte) 1)
#define Negative ((Byte) 0)
#define CW ((Byte) 2)
#define CCW ((Byte) 1)
#define NoRotation ((Byte) 0)

/* screen window border defines... */

/* single line */
#define HorizBorder       196
#define VertBorder        179
#define TopLeftCorner     218
#define TopRightCorner    191
#define BottomLeftCorner  192
#define BottomRightCorner 217
#define LeftIntersect     195
#define RightIntersect    180

#define DblHorizBorder       205
#define DblVertBorder        186
#define DblTopLeftCorner     201
#define DblTopRightCorner    187
#define DblBottomLeftCorner  200
#define DblBottomRightCorner 188
#define DblLeftIntersect     199
#define DblRightIntersect    182

/* keyboard defines... */

#define Backspace 8
#define Tab 9
#define Return 13
#define Esc 27
#define Blank 32

/* extended keyboard defines... */

#define ExtendedKeyboardStroke 0

#define UpCursor    72
#define DownCursor  80
#define LeftCursor  75
#define RightCursor 77
#define Shift_Tab   15
/* the left aprostrophe as found in upper left corner of most keyboards */
#define Left_Apostrophe 96
#define EqualsKey 61

#define F1  59
#define F2  60
#define F3  61
#define F4  62
#define F5  63
#define F6  64
#define F7  65
#define F8  66
#define F9  67
#define F10 68
#define F11 133
#define F12 134

#define Shift_F1  84
#define Shift_F2  85
#define Shift_F3  86
#define Shift_F4  87
#define Shift_F5  88
#define Shift_F6  89
#define Shift_F7  90
#define Shift_F8  91
#define Shift_F9  92
#define Shift_F10 93

#define Ctrl_F1  94
#define Ctrl_F2  95
#define Ctrl_F3  96
#define Ctrl_F4  97
#define Ctrl_F5  98
#define Ctrl_F6  99
#define Ctrl_F7  100
#define Ctrl_F8  101
#define Ctrl_F9  102
#define Ctrl_F10 103

#define Alt_F1  104
#define Alt_F2  105
#define Alt_F3  106
#define Alt_F4  107
#define Alt_F5  108
#define Alt_F6  109
#define Alt_F7  110
#define Alt_F8  111
#define Alt_F9  112
#define Alt_F10 113

#define HeadKeysMemoryLocation 0x0040001AL
#define TailKeysMemoryLocation 0x0040001CL
#define KeyStroke (*headkeys != *tailkeys)

/* pointers to memory areas holding start, end of keystrokes */
int far *headkeys;
int far *tailkeys;

/* common message define... */

#define ContMsg "Press any key to continue..."

/* common conversion variables... */

double OneRev;
double HalfRev;
double QtrRev;
double RadToDeg;
double RadToArcmin;
double RadToArcsec;
double RadToTenthsArcsec;
double RadToHr;
double RadToMin;
double RadToSec;
double RadToHundSec;
double DaysToHr;
double DaysToMin;
double DaysToSec;
double ArcsecPerRev;
double ArcsecPerDeg;

double SidRate;

double ClockTicksDay;
double ClockTicksSec;
double ClockTicksMin;
double ClockTicksHr;
double RadToClockTick;

void ContMsgRoutine( void);
void BadMsgExit( const char* Msg);
void InitCommonVars( void);

/* keyboard response or input */
int Response;

#define NameSize  40
#define ValueSize 40
#define ObjectNameSize 20

#define NewLine printf(" \n")
#define NameBlanks "                    "
#define EmptyString ""

#define MaxCharsInt 4
#define MaxCharsDbl 7

struct ffblk Ffblk;
Flag FindFirst;
Flag FoundFile;

char Name[NameSize];
char Value[ValueSize];

int GetNumStr( char NumStr[], int NumChars);
int GetInt( int* Num);
int GetDouble( double* Num);

FILE* Input;
FILE* Output;

void ReadDouble( FILE* File, double* V);
void ReadFlag( FILE* File, Flag* V);
void ReadByte( FILE* File, Byte* V);
void ReadInt( FILE* File, int* V);
void ReadUnsigned( FILE* File, unsigned* V);
void ReadLong( FILE* File, long* V);
void FReadChar( FILE* File, char* V);
void FReadToNewLine( FILE* File);
void FReadToChar( FILE* File, const char V);
void ReadString( FILE* File, char* Name);
int FindNextIncrFilename( char* NameStart, char* Extension);

/* serial port... */

/* for ReadSerial() to work, SerBufEnd must be 2^n - 1 */
#define SerBufEnd 1023

#define Com1Base 0x3F8
#define Com2Base 0x2F8
#define Com3Base 0x3E8
#define Com4Base 0x2E8

#define IER 1 		/* interrupt enable register */
#define LCR 3		/* line control register */
#define MCR 4		/* modem control register */
#define LSR 5		/* line status register */
#define MSR 6 		/* modem status register */
#define MCR_DTR 1 /* bit to turn on DTR */
#define MCR_RTS 2 /* bit to turn on RTS */

#define Int4Mask 0x10 /* bit 5, ie, 10000 */
#define Int3Mask 0x08 /* bit 4, ie, 1000 */
#define Com1Int 0x0C  /* IRQ 4 */
#define Com2Int 0x0B  /* IRQ 3 */
#define Com3Int 0x0C  /* IRQ 4 */
#define Com4Int 0x0B  /* IRQ 3 */

#define IntMaskReg 0x21
#define PIC_EOI_ADDR 0x20
#define EOI 0x20

#define NoneParity 0
#define OddParity 8
#define EvenParity 24
#define MarkParity 40
#define SpaceParity 56

#define CR 13
#define LF 10
#define SI 15
#define SY 22
#define D2 18

/* count of chars in buffer; buffer index goes from 0 to SerBufEnd, so array
size is SerBufEnd + 1 */
volatile Byte Ser1Buf[SerBufEnd+1];
unsigned Ser1BufRdIx;
volatile unsigned Ser1BufWrIx;
volatile unsigned int Ser1Count;
volatile Byte Ser2Buf[SerBufEnd+1];
unsigned Ser2BufRdIx;
volatile unsigned Ser2BufWrIx;
volatile unsigned int Ser2Count;
unsigned SerialBase;
unsigned BaudRate;
Byte Parity;
Byte DataBits;
Byte StopBits;

void interrupt( *OldSer1IntVec)();
void interrupt( *OldSer2IntVec)();
void interrupt Ser1ISR( void);
void interrupt Ser2ISR( void);
void InitSerial( const int ComPort, const int BaudRate, const int Parity,
const int DataBits, const int StopBits);
void CloseSerial( const int ComPort);
void InitRingBuffer( const int ComPort);
Flag ReadSerial( const int ComPort, Byte* B);
Flag WriteSerial( const int ComPort, Byte B);
Flag WriteSerialString( const int ComPort, char C[], const int CharCount);
void TestSerial( const int ComPort);

/* video... */

/* as in 'text' element, analagous to 'pixel' = picture element */
struct Texel
{
	char Ch;
	unsigned char Attr;
};

/* Using this structure we can define a two - dimensional array to correspond
to the memory arrangement of the screen where X is across or columns and Y is
up and down or rows (X, Y are backwards from the x, y in goto( x, y)) */

#define MaxY 25
#define MaxX 80

/* ScreenArea is type defined as a two dimensional array of 'Texel's */
typedef struct Texel ScreenArea[MaxY][MaxX];

#define Screen (*ScreenPtr)

struct XY
{
	int X, Y;
};

char StrBuf[MaxX];
char StrBuf2[MaxX];

/* X = across or columns, and Y = up and down or rows */
struct XY VidMemXY;
int TextAttr;
unsigned MemSeg, MemOff;
ScreenArea far* ScreenPtr;
union REGS Regs;

void InitVideo( const Flag DisplayOpeningMsgs);
void WriteCharToScreen( const char Char);
void WriteStrBufToScreen( void);
void TestVideo( void);

/* CMOS clock... */

#define CMOS_Ctrl_Port 0x70
#define CMOS_Rtn_Port  0x71

#define CCP      CMOS_Ctrl_Port
#define CMOS_RTN inport( CMOS_Rtn_Port)
/* BCD = binary coded decimal; if 0xAB, then 'A' is tens and 'B' is ones in
binary */
#define CMOS_DECODE_BCD (int) (((BCD & 0xF0)>>4)*10 + (BCD & 0x0F))
#define ENCODE_BCD (int) (((BCD/10)<<4) + BCD - (BCD/10)*10) 

#define CMOS_Reg_BCD_Sec   0
#define CMOS_Reg_BCD_Min   2
#define CMOS_Reg_BCD_Hr    4
#define CMOS_Reg_BCD_Day   7
#define CMOS_Reg_BCD_Mon   8
#define CMOS_Reg_BCD_Yr    9
#define CMOS_Reg_Status_A 10

#define Update_In_Progress 128

struct time t;
struct date d;

/* vars for CMOS clock... */

int DOS_Yr;
int DOS_Mon;
int DOS_Day;
int DOS_Hr;
int DOS_Min;
int DOS_Sec;

int CMOS_Status_A;
int BCD;
int Hold_BCD;
int CMOS_Sec;
int CMOS_Min;
int CMOS_Hr;
int CMOS_Day;
int CMOS_Mon;
int CMOS_Yr;
int Century;

int UpdateInProgress( void);
void DisplayCMOSTimeDate( void);
void SetDOSToCMOS( void);
void SetRealTimeClock( const int hr, const int min, const int sec);
void SetDateofRealTimeClock( const int century, const int year, const int mon,
const int day);

/* astronomical times... */

#define ETZ 5
#define TICK_ADDR 0x0040006CL

struct HMSH
{
	int Sign;
	int Hr;
	int Min;
	int Sec;
	int HundSec;
};

struct DMS
{
	int Sign;
	int Deg;
	int Min;
	int Sec;
};

/* Ra, Dec, Alt, Az, SidT kept in radians */
struct Position
{
	double Ra;
	struct HMSH RaHMSH;
	double Dec;
	struct DMS DecDMS;
	double HA;
	struct HMSH HAHMSH;
	double Alt;
	double Az;
	double SidT;
	struct HMSH SidTimeHMSH;
	Flag Init;
};

/* pointer to bios memory area holding bios clock tick count */
long far* Ticks;
long StartTicks;
int Days;
double JD;
double SidHr;
double SidT;
double StartSidT;

struct HMSH SidTimeHMSH;

/* current coordinates */
/* define here bec. of NewSidT(); */
struct Position Current;

void InitTimes(  const Flag DisplayOpeningMsgs, const int Tz, const int DST,
double LongitudeDeg);

void SetStartBiosClockAndSidTime( const long Yr, const int M, const int D,
const int h, const int m, const double s, const int Tz, const int DST,
double LongitudeDeg);

int NewSidT( void);

void CalcJD( const long Y, const int M, const int D, const int Tz,
const int DST, const int h, const int m, const double s);

void CalcSidHr( const long Y, const int M, const int D, const int Tz,
const int DST, const int h, const int m, const double s, const double LongitudeDeg);

void GetHMSH( long TotalHundSec, struct HMSH* V);
void DisplayHMSH( const struct HMSH V);
void StrBufHMSH( const struct HMSH V);
void DisplayHMST( const struct HMSH V);
void DisplayHMS( const struct HMSH V);
void StrBufHMS( const struct HMSH V);
void DisplaySHMS( const struct HMSH V);
void StrBufSHMS( const struct HMSH V);
void CalcRadFromHMSH( double* R, struct HMSH V);
void CalcRadFromDMS( double* R, struct DMS V);

void TestTimes( void);

/* coordinates... */

#define CurrentFile "CURRENT.DAT"
#define SizeofHMSX 8
#define SizeofDMSX 9
#define SizeofDegX 7

void InitCoord( const Flag DisplayOpeningMsgs);
void SaveCurrentAltaz( void);
void LoadCurrentAltaz( const Flag DisplayOpeningMsgs);
void ValidRa( struct Position* Pos);
void ValidHA( struct Position* Pos);
void ValidDec( struct Position* Pos);
void ValidAlt( struct Position* Pos);
void ValidAz( struct Position* Pos);

void SetCoord( struct Position* Pos, const double RaDeg, const double DecDeg,
const double AltDeg, const double AzDeg, const double SidTDeg);

void ShowCoord( struct Position* Pos);
void GetDMS( long TotalSec, struct DMS* V);
void DisplayDMS( const struct DMS D);
void StrBufDMS( const struct DMS D);
void VidMemDeg( const double Deg);
void VidMemRaHMS( struct Position* Pos);
void VidMemRaHMSH( struct Position* Pos);
void VidMemHAHMS( struct Position* Pos);
void VidMemDecDMS( struct Position* Pos);
void VidMemSidT( struct Position* Pos);

/* parallel port... */

unsigned PPortAddr;
unsigned PPortAddrOutByte;
unsigned PPortAddrInNibble;
unsigned PPortAddrInOutNibble;
Byte OutByteValue;
Byte BiDirOutNibbleValue;

void InitPPort( void);
unsigned GetPPortAddr( int lpt);
void DisplayLpts (void);
void OutByte( void);
void BiDirOutNibble( void);
Byte InNibble( void);
Byte BiDirInNibble( void);
void TestPPort( void);

/* refraction... */

#define MaxRefractIx 5

double Refract; 

/* to correct for refraction, add interpolation of R1 table to altitude */
double R1[MaxRefractIx+1][2];
/* to remove refraction, subtract interpolation of R2 table from altitude */
double R2[MaxRefractIx+1][2];

void InitRefract( void);
void CalcRefractScopeToSky( const double Alt);
void CalcRefractSkyToScope( const double Alt); 

/* egavga routines */

int color;
int maxx, maxy;

/* to save screen window */
char WinBuffer[MaxX*MaxY*2];

void InitGraphics( void);
void CloseGraphics( void);

/* mouse functions */

#define MOUSE_INTERRUPT 0x33

#define RESET_MOUSE 0
#define DISPLAY_MOUSE 1
#define HIDE_MOUSE 2
#define GET_MOUSE_POSITION 3
#define GET_MOUSE_POSITION_RELATIVE 0x0B
#define SET_MOUSE_POSITION 4
#define GET_MOUSE_PRESS_INFO 5
#define GET_MOUSE_RELEASE_INFO 6
#define SET_MOUSE_SENSITIVITY 0x1A
#define GET_MOUSE_SENSITIVITY 0x1B

#define SUCCESSFULL_MOUSE_RESET -1

#define LEFT_BUTTON 0
#define RIGHT_BUTTON 1

#define MaxXText 80
#define MaxYText 25
#define XYTextRatio (MaxXText/MaxYText)
#define MouseMaxXText (MaxXText-1)
#define MouseMaxYText (MaxYText-1)
#define MouseMidXText (MaxXText/2)
#define MouseMidYText (MaxYText/2)

int inax;
int inbx;
int incx;
int indx;
int outax;
int outbx;
int outcx;
int outdx;
int HorizMickeySensitivity;
int VertMickeySensitivity;
int ThresholdDblSpeed;
int MouseXGraph;
int MouseYGraph;
int MouseXMickeyRelative;
int MouseYMickeyRelative;
int MouseXText;
int MouseYText;

void InitMouse( void);
void CallMouse( void);
void GetDefaultMouseSensitivity( void);
/* this function decodes cx and dx into horiz and vert positions;
call CallMouse() first */
void DecodeMousePosition( void);
void ResetMouse( void);
void LowMouseSensitivity( void);
void ResetMouseSensitivity( void);
void DisplayMouse( void);
void HideMouse( void);
void GetMousePosition( void);
void GetMousePositionRelative( void);
// mouse stays hidden if HideMouse() called previously
void SetMousePosition(int Horiz, int Vert);
void CenterMouseCursor( void);
int MouseLeftButtonPress( void);
int MouseRightButtonPress( void);
int MouseLeftButtonReleased( void);
int MouseRightButtonReleased( void);
void TestMouse( void);

#endif
